     /**
      * \brief Check Digit Routines
      *
      * This service program will cover various check digit algorithms (f. e.
      * EAN-8, EAN-13, etc.).
      *
      * \author Mihael Schmidt
      * \date   01.01.2011
      *
      * \link http://commons.apache.org/validator/ Apache Commons Validator Framework
      */
 
      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of the VALIDATOR service program.
      *
      * VALIDATOR is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * VALIDATOR is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with VALIDATOR.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------
 
     H nomain
 
      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------     
     D calculateModulus...
     D                 PR            10I 0
     D   code                       128A   const varying
     D   includesCheckDigit...
     D                                 N   const
     D   modulus                     10I 0 const
      *
     D weightedValue...
     D                 PR            10I 0
     D charValue                     10I 0 const
     D leftPos                       10I 0 const
     D rightPos                      10I 0 const
 
 
      /include 'message_h.rpgle'
      /include 'checkdigit_h.rpgle'
 
 
     /**
      * \brief Validate code
      *
      * Validate a modulus check digit for a code.
      *
      * \param Code
      * \param Modulus (default: 10)
      *
      * \return *on = code is valid <br>
      *         *off = no code passed or code invalid
      */
     P rng_validator_modulus_isValid...
     P                 B                   export
     D                 PI              N
     D  code                        128A   const varying
     D  pModulus                     10I 0 const options(*nopass)
      *
     D modulusResult   S             10I 0
     D modulus         S             10I 0 inz(10)
      /free
       if (code = *blank);
         return *off;
       endif;
 
       if (%parms() = 2);
         modulus = pModulus;
       endif;
 
       monitor;
         modulusResult = calculateModulus(code : *on : modulus);
         return (modulusResult = 0);
 
         on-error *all;
           return *off;
       endmon;
      /end-free
     P                 E
 
 
     /**
      * \param Calculate check digit
      * 
      * Calculates the check digit for the passed code.
      * 
      * \param Code (without check digit)
      * 
      * \return calculated check digit
      * 
      * \throws CPF9898 Invalid check digit
      */
     P rng_validator_modulus_calculate...
     P                 B                   export
     D                 PI             1P 0
     D   code                       128A   const varying
     D   pModulus                    10I 0 const options(*nopass)
      *
     D modulus         S             10I 0 inz(10)
     D modulusResult   S             10I 0
     D checkDigit      S             10I 0
      /free
       if (%parms() = 2);
         modulus = pModulus;
       endif;
 
       modulusResult = calculateModulus(code : *off : modulus);
       checkDigit = modulus - modulusResult;
 
       if (checkDigit = modulus);
         checkDigit = 0;
       endif;
 
       if (checkDigit >= 0 and checkDigit <= 9);
         return checkDigit;
       else;
         msg_sendEscapeMessage('Invalid check digit value '+ %char(checkDigit));
         return 0;
       endif;
      /end-free
     P                 E
 
 
     /**
      * \brief Calculate the modulus for a code.
      *
      * \param The code to calculate the modulus for.
      * \param Whether the code includes the Check Digit or not.
      *
      * \return The modulus value
      *
      * \throws if an error occurs calculating the modulus for the specified code
      */
     P calculateModulus...
     P                 B
     D                 PI            10I 0
     D   pCode                      128A   const varying
     D   includesCheckDigit...
     D                                 N   const
     D   modulus                     10I 0 const
      *
     D code            S            128A   varying
     D codePtr         S               *
     D total           S             10I 0
     D i               S             10I 0
     D length          S             10I 0
     D leftPos         S             10I 0
     D rightPos        S             10I 0
     D charValue       S             10I 0
     D char            S              1A   based(ptr)
     D offset          S              3I 0
      /free
       code = pCode;
       codePtr = %addr(code : *DATA);
       length = %len(%trim(code));
 
       if (not includesCheckDigit);
         offset = 1;
       endif;
 
       for i = 0 to length-1;
         ptr = codePtr + i;
         leftPos  = i + 1;
         rightPos = length - i + offset;
         charValue = %int(char);
         total += weightedValue(charValue : leftPos : rightPos);
       endfor;
 
       if (total = 0);
         msg_sendEscapeMessage('Invalid code, sum is zero');
       endif;
 
       return %rem(total : modulus);
      /end-free
     P                 E
 
 
     /**
      * \brief Calculate weight of number
      *
      * <p>Calculates the <i>weighted</i> value of a character in the
      * code at a specified position.</p>
      *
      * <p>For EAN-13 (from right to left) <b>odd</b> digits are weighted
      * with a factor of <b>one</b> and <b>even</b> digits with a factor
      * of <b>three</b>.</p>
      *
      * \param The numeric value of the character.
      * \param The position of the character in the code, counting from left to right (1-based)
      * \param The position of the character in the code, counting from right to left (1-based)
      *
      * \return The weighted value of the character.
      */
     P weightedValue   B
     D                 PI            10I 0
     D charValue                     10I 0 const
     D leftPos                       10I 0 const
     D rightPos                      10I 0 const
      *
     D weight          S             10I 0
     D weights         DS
     D   pos_weight_1                10I 0 inz(3)
     D   pos_weight_2                10I 0 inz(1)
     D   pos_weights                 10I 0 dim(2) overlay(weights)
      /free
       weight = pos_weights(%rem(rightPos : 2) + 1);
       return charValue * weight;
      /end-free
     P                 E
